@@ -5,7 +5,7 @@
  *  Mike Gemuende <mike@gemuende.de>
  *  Jim Nelson <jim@yorba.org>
  *
- * This is free software. See COPYING for details.
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "gexiv2-metadata.h"
@@ -23,6 +23,10 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
+#ifdef G_OS_WIN32
+#include <glib/gwin32.h>
+#endif
+
 #include <exiv2/exiv2.hpp>
 
 #if EXIV2_TEST_VERSION(0,27,99)
@@ -62,7 +66,7 @@ public:
     using ptr_type = Exiv2::BasicIo::AutoPtr;
 #endif
 
-    int open() {
+    int open() override {
         if (_seekable == nullptr)
             return 0;
 
@@ -77,17 +81,15 @@ public:
         return 0;
     }
 
-    int close() {
-        return 0;
-    }
+    int close() override { return 0; }
 
     // Writing is not supported
-    size_type write(const Exiv2::byte *data, size_type wcount) { return 0; }
-    size_type write(BasicIo &src) { return 0; }
-    int putb(Exiv2::byte data) { return EOF; }
+    size_type write(const Exiv2::byte* /*data*/, size_type /*wcount*/) override { return 0; }
+    size_type write(BasicIo& /*src*/) override { return 0; }
+    int putb(Exiv2::byte /*data*/) override { return EOF; }
 
 #if EXIV2_TEST_VERSION(0,27,99)
-    Exiv2::DataBuf read(size_t rcount) noexcept {
+    Exiv2::DataBuf read(size_t rcount) noexcept override {
 #else
     Exiv2::DataBuf read(long rcount) override {
 #endif
@@ -101,7 +103,7 @@ public:
         return b;
     }
 
-    size_type read(Exiv2::byte *buf, size_type rcount) {
+    size_type read(Exiv2::byte* buf, size_type rcount) override {
         GError *error = NULL;
         gssize result = 0;
 
@@ -128,16 +130,16 @@ public:
         return result;
     }
 
-    int getb() {
+    int getb() override {
         Exiv2::byte b;
         return this->read (&b, 1) == 1 ? b : EOF;
     }
 
-    void transfer(Exiv2::BasicIo &src) {
+    void transfer(Exiv2::BasicIo& /*src*/) override {
         // Does not seem necessary for Read-only support
     }
 
-    int seek(seek_offset_t offset, Exiv2::BasicIo::Position position) {
+    int seek(seek_offset_t offset, Exiv2::BasicIo::Position position) override {
         if (_seekable != NULL && g_seekable_can_seek (_seekable)) {
             GSeekType t = G_SEEK_SET;
             switch (position) {
@@ -186,44 +188,30 @@ public:
         }
     }
 
-    Exiv2::byte *mmap(bool writable) {
-        return NULL;
-    }
+    Exiv2::byte* mmap(bool /*writable*/) override { return nullptr; }
 
-    int munmap() {
-        return 0;
-    }
+    int munmap() override { return 0; }
 
-    long tell() const {
-        if (_seekable != NULL && g_seekable_can_seek (_seekable)) {
+    long tell() const override {
+        if (_seekable != nullptr && g_seekable_can_seek (_seekable)) {
             return static_cast<long>(g_seekable_tell (_seekable));
         } else {
             return -1;
         }
     }
 
-    size_t size() const {
-        return static_cast<size_t>(_size);
-    }
+    size_t size() const override { return static_cast<size_t>(_size); }
 
-    bool isopen() const {
-        return true;
-    }
+    bool isopen() const override { return true; }
 
-    int error() const {
-        return _error == nullptr ? 0 : 1;
-    }
+    int error() const override { return _error == nullptr ? 0 : 1; }
 
-    bool eof() const {
-        return _eof;
-    }
+    bool eof() const override { return _eof; }
 
-    std::string path() const {
-        return "GIO Wrapper";
-    }
+    std::string path() const override { return "GIO Wrapper"; }
 
 #ifdef EXV_UNICODE_PATH
-    std::wstring wpath() const {
+    std::wstring wpath() const override {
         std::string p = path();
         std::wstring w(p.length(), L' ');
         std::copy(p.begin(), p.end(), w.begin());
@@ -250,6 +238,46 @@ private:
 }; // class GioIo
 } // Anonymous namespace
 
+// -----------------------------------------------------------------------------
+// Misc internal helper functions
+
+// Port of the NaturalCollate.vala collation key generation to C++
+// Simplified to assume that all XMP keys are ASCII and not UTF-8
+// Original source:
+// https://gitlab.gnome.org/GNOME/shotwell/-/blob/master/src/NaturalCollate.vala
+std::string detail::collate_key(const std::string& str) {
+    constexpr char SUPERDIGIT = ':';
+    constexpr char COLLATION_SENTINAL[] = "\x01\x01\x01";
+    constexpr char NUM_SENTINEL = 0x2;
+
+    std::stringstream in{str};
+    std::stringstream out{};
+
+    while (not in.eof()) {
+        // As long as there are no digits, we put them from input to output
+        while (not std::isdigit(in.peek()) && not in.eof()) {
+            out << static_cast<char>(in.get());
+        }
+
+        if (not in.eof()) {
+
+            // We read the number (integer only)...
+            uint64_t number;
+            in >> number;
+
+            std::string to_append(std::to_string(number).length(), SUPERDIGIT);
+
+            // ... and append it together with its length in : to the output
+            out << COLLATION_SENTINAL << NUM_SENTINEL << to_append << number;
+        }
+    }
+
+    // Add a sentinal for good measure (no idea, follows the original code)
+    out << NUM_SENTINEL;
+
+    return out.str();
+}
+
 G_BEGIN_DECLS
 
 G_DEFINE_TYPE_WITH_CODE (GExiv2Metadata, gexiv2_metadata, G_TYPE_OBJECT, G_ADD_PRIVATE (GExiv2Metadata));
@@ -261,9 +289,8 @@ static gboolean gexiv2_metadata_open_internal (GExiv2Metadata *self, GError **er
 static gboolean gexiv2_metadata_save_internal (GExiv2Metadata *self, image_ptr image, GError **error);
 
 static void gexiv2_metadata_init (GExiv2Metadata *self) {
-    GExiv2MetadataPrivate *priv;
-    self->priv = priv = (GExiv2MetadataPrivate *) gexiv2_metadata_get_instance_private(self);
-    
+    self->priv = (GExiv2MetadataPrivate*) gexiv2_metadata_get_instance_private(self);
+
     /* Initialize members */
     self->priv->comment = NULL;
     self->priv->mime_type = NULL;
@@ -376,12 +403,58 @@ static gboolean gexiv2_metadata_open_internal (GExiv2Metadata* self, GError** er
     return TRUE;
 }
 
-gboolean gexiv2_metadata_open_path (GExiv2Metadata *self, const gchar *path, GError **error) {
-    g_return_val_if_fail (GEXIV2_IS_METADATA (self), FALSE);
-    
+#ifdef EXV_UNICODE_PATH
+static std::wstring convert_path(const char* path, GError** error) {
+    wchar_t* wfile{nullptr};
+    wfile = reinterpret_cast<wchar_t*>(g_utf8_to_utf16(path, -1, nullptr, nullptr, error));
+    // Error is set by g_utf8_to_utf16()
+    if (wfile == nullptr) {
+        return {};
+    }
+
+    std::wstring file{wfile};
+    g_free(wfile);
+
+    return file;
+}
+#else
+static std::string convert_path(const char* path, GError** error) {
+#ifdef G_OS_WIN32
+    std::string file;
+    char* local_path = g_win32_locale_filename_from_utf8(path);
+    if (local_path == nullptr) {
+        char* msg = g_strdup_printf("Failed to convert \"%s\" to locale \"%s\"", path, g_win32_getlocale());
+        g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_INVALID_FILENAME, msg);
+
+        g_free(msg);
+
+        return FALSE;
+    }
+    file = local_path;
+    g_free(local_path);
+
+    return file;
+#else
+    g_return_val_if_fail(error != nullptr && *error == nullptr, std::string());
+
+    return std::string{path};
+#endif
+}
+#endif
+
+gboolean gexiv2_metadata_open_path(GExiv2Metadata* self, const gchar* path, GError** error) {
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
     try {
-        self->priv->image = Exiv2::ImageFactory::open (path);
-        
+        GError* inner_error = nullptr;
+
+        auto converted_path = convert_path(path, &inner_error);
+        if (inner_error != nullptr) {
+            g_propagate_error(error, inner_error);
+
+            return FALSE;
+        }
+        self->priv->image = Exiv2::ImageFactory::open(converted_path);
+
         return gexiv2_metadata_open_internal (self, error);
     } catch (Exiv2::Error &e) {
         g_set_error_literal (error, g_quark_from_string ("GExiv2"), e.code (), e.what ());
@@ -552,7 +625,18 @@ gboolean gexiv2_metadata_save_external (GExiv2Metadata *self, const gchar *path,
     g_return_val_if_fail (GEXIV2_IS_METADATA (self), FALSE);
 
     try {
-        return gexiv2_metadata_save_internal (self, Exiv2::ImageFactory::create(Exiv2::ImageType::xmp, path), error);
+        GError* inner_error = nullptr;
+
+        auto local_path = convert_path(path, &inner_error);
+        if (inner_error != nullptr) {
+            g_propagate_error(error, inner_error);
+
+            return FALSE;
+        }
+
+        return gexiv2_metadata_save_internal(self,
+                                             Exiv2::ImageFactory::create(Exiv2::ImageType::xmp, local_path),
+                                             error);
     } catch (Exiv2::Error &e) {
         g_set_error_literal (error, g_quark_from_string ("GExiv2"), e.code (), e.what ());
     }
@@ -564,7 +648,16 @@ gboolean gexiv2_metadata_save_file (GExiv2Metadata *self, const gchar *path, GEr
     g_return_val_if_fail (GEXIV2_IS_METADATA (self), FALSE);
     
     try {
-        return gexiv2_metadata_save_internal (self, Exiv2::ImageFactory::open (path), error);
+        GError* inner_error = nullptr;
+
+        auto local_path = convert_path(path, &inner_error);
+        if (inner_error != nullptr) {
+            g_propagate_error(error, inner_error);
+
+            return FALSE;
+        }
+
+        return gexiv2_metadata_save_internal(self, Exiv2::ImageFactory::open(local_path), error);
     } catch (Exiv2::Error &e) {
         g_set_error_literal (error, g_quark_from_string ("GExiv2"), e.code (), e.what ());
     }
@@ -652,7 +745,7 @@ GExiv2Orientation gexiv2_metadata_get_orientation (GExiv2Metadata *self) {
         // Because some camera set a wrong standard exif orientation tag,
         // We need to check makernote tags first!
         if (gexiv2_metadata_has_exif_tag(self, "Exif.MinoltaCs7D.Rotation")) {
-            long orientation = gexiv2_metadata_get_exif_tag_long(self, "Exif.MinoltaCs7D.Rotation");
+            long orientation = gexiv2_metadata_get_exif_tag_long(self, "Exif.MinoltaCs7D.Rotation", nullptr);
             switch (orientation) {
                 case 76:
                     return GEXIV2_ORIENTATION_ROT_90;
@@ -666,7 +759,7 @@ GExiv2Orientation gexiv2_metadata_get_orientation (GExiv2Metadata *self) {
         }
 
         if (gexiv2_metadata_has_exif_tag(self, "Exif.MinoltaCs5D.Rotation")) {
-            long orientation = gexiv2_metadata_get_exif_tag_long(self, "Exif.MinoltaCs5D.Rotation");
+            long orientation = gexiv2_metadata_get_exif_tag_long(self, "Exif.MinoltaCs5D.Rotation", nullptr);
             switch (orientation) {
                 case 76:
                     return GEXIV2_ORIENTATION_ROT_90;
@@ -681,7 +774,7 @@ GExiv2Orientation gexiv2_metadata_get_orientation (GExiv2Metadata *self) {
         }
         
         GExiv2Orientation orientation = (GExiv2Orientation) gexiv2_metadata_get_exif_tag_long(self,
-            "Exif.Image.Orientation");
+            "Exif.Image.Orientation", nullptr);
         if (orientation >= GEXIV2_ORIENTATION_MIN && orientation <= GEXIV2_ORIENTATION_MAX)
             return orientation;
     }
@@ -689,7 +782,7 @@ GExiv2Orientation gexiv2_metadata_get_orientation (GExiv2Metadata *self) {
     GExiv2Orientation orientation = GEXIV2_ORIENTATION_UNSPECIFIED;
     if (gexiv2_metadata_has_xmp_tag(self, "Xmp.tiff.ImageWidth")) {
         orientation = (GExiv2Orientation) gexiv2_metadata_get_xmp_tag_long(self,
-            "Xmp.tiff.ImageWidth");
+            "Xmp.tiff.ImageWidth", nullptr);
     }
     
     return (orientation < GEXIV2_ORIENTATION_MIN || orientation > GEXIV2_ORIENTATION_MAX)
@@ -753,18 +846,18 @@ gint gexiv2_metadata_get_metadata_pixel_width (GExiv2Metadata *self) {
     
     if (gexiv2_metadata_has_exif(self)) {
         if (gexiv2_metadata_has_exif_tag(self, "Exif.Photo.PixelXDimension"))
-            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Photo.PixelXDimension");
+            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Photo.PixelXDimension", nullptr);
         
         if (gexiv2_metadata_has_exif_tag(self, "Exif.Image.ImageWidth"))
-            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Image.ImageWidth");
+            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Image.ImageWidth", nullptr);
     }
     
     if (gexiv2_metadata_has_xmp(self)) {
         if (gexiv2_metadata_has_xmp_tag(self, "Xmp.tiff.ImageWidth"))
-            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.tiff.ImageWidth");
+            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.tiff.ImageWidth", nullptr);
         
         if (gexiv2_metadata_has_xmp_tag(self, "Xmp.exif.PixelXDimension"))
-            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.exif.PixelXDimension");
+            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.exif.PixelXDimension", nullptr);
     }
     
     return -1;
@@ -776,18 +869,18 @@ gint gexiv2_metadata_get_metadata_pixel_height (GExiv2Metadata *self) {
     
     if (gexiv2_metadata_has_exif(self)) {
         if (gexiv2_metadata_has_exif_tag(self, "Exif.Photo.PixelYDimension"))
-            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Photo.PixelYDimension");
+            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Photo.PixelYDimension", nullptr);
         
         if (gexiv2_metadata_has_exif_tag(self, "Exif.Image.ImageLength"))
-            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Image.ImageLength");
+            return gexiv2_metadata_get_exif_tag_long(self, "Exif.Image.ImageLength", nullptr);
     }
     
     if (gexiv2_metadata_has_xmp(self)) {
         if (gexiv2_metadata_has_xmp_tag(self, "Xmp.tiff.ImageHeight"))
-            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.tiff.ImageHeight");
+            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.tiff.ImageHeight", nullptr);
             
         if (gexiv2_metadata_has_xmp_tag(self, "Xmp.exif.PixelYDimension"))
-            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.exif.PixelYDimension");
+            return gexiv2_metadata_get_xmp_tag_long(self, "Xmp.exif.PixelYDimension", nullptr);
     }
     
     return -1;
@@ -827,37 +920,37 @@ gchar* gexiv2_metadata_get_comment (GExiv2Metadata *self) {
     if (str != NULL && *str != '\0')
         return g_strdup(str);
     
-    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Image.ImageDescription");
+    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Image.ImageDescription", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
         g_free (str);
     
-    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Photo.UserComment");
+    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Photo.UserComment", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
         g_free (str);
     
-    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Image.XPComment");
+    str = gexiv2_metadata_get_exif_tag_interpreted_string (self, "Exif.Image.XPComment", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
         g_free (str);
     
-    str = gexiv2_metadata_get_iptc_tag_interpreted_string (self, "Iptc.Application2.Caption");
+    str = gexiv2_metadata_get_iptc_tag_interpreted_string (self, "Iptc.Application2.Caption", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
         g_free (str);
     
-    str = gexiv2_metadata_get_xmp_tag_interpreted_string (self, "Xmp.dc.description");
+    str = gexiv2_metadata_get_xmp_tag_interpreted_string (self, "Xmp.dc.description", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
         g_free (str);
     
-    str = gexiv2_metadata_get_xmp_tag_interpreted_string (self, "Xmp.acdsee.notes");
+    str = gexiv2_metadata_get_xmp_tag_interpreted_string (self, "Xmp.acdsee.notes", NULL);
     if (str != NULL && *str != '\0')
         return str;
     else
@@ -911,116 +1004,255 @@ gboolean gexiv2_metadata_is_iptc_tag(const gchar* tag) {
     return g_ascii_strncasecmp("Iptc.", tag, 5) == 0;
 }
 
-gchar* gexiv2_metadata_get_tag_string (GExiv2Metadata *self, const gchar* tag) {
-    g_return_val_if_fail(GEXIV2_IS_METADATA (self), NULL);
-    g_return_val_if_fail(tag != NULL, NULL);
-    g_return_val_if_fail(self->priv->image.get() != NULL, NULL);
+gchar* gexiv2_metadata_try_get_tag_string (GExiv2Metadata *self, const gchar* tag, GError **error) {
+    g_return_val_if_fail(GEXIV2_IS_METADATA (self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_string (self, tag);
+        return gexiv2_metadata_get_xmp_tag_string (self, tag, error);
         
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_string (self, tag);
+        return gexiv2_metadata_get_exif_tag_string (self, tag, error);
         
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_string (self, tag);
+        return gexiv2_metadata_get_iptc_tag_string (self, tag, error);
     
     return NULL;
 }
 
+gboolean gexiv2_metadata_try_set_tag_string (GExiv2Metadata *self, const gchar* tag, const gchar* value, GError **error) {
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+    
+    if (gexiv2_metadata_is_xmp_tag(tag))
+        return gexiv2_metadata_set_xmp_tag_string(self, tag, value, error);
+    
+    if (gexiv2_metadata_is_exif_tag(tag))
+        return gexiv2_metadata_set_exif_tag_string(self, tag, value, error);
+    
+    if (gexiv2_metadata_is_iptc_tag(tag))
+        return gexiv2_metadata_set_iptc_tag_string(self, tag, value, error);
+    
+    return FALSE;
+}
+
+gchar* gexiv2_metadata_get_tag_string (GExiv2Metadata *self, const gchar* tag) {
+    gchar  *value;
+    GError *error = nullptr;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA (self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_string (self, tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
 gboolean gexiv2_metadata_set_tag_string (GExiv2Metadata *self, const gchar* tag, const gchar* value) {
+    gboolean  success;
+    GError   *error = nullptr;
+
     g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
-    g_return_val_if_fail(tag != NULL, FALSE);
-    g_return_val_if_fail(self->priv->image.get() != NULL, FALSE);
-    
-    if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_set_xmp_tag_string(self, tag, value);
-    
-    if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_set_exif_tag_string(self, tag, value);
-    
-    if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_set_iptc_tag_string(self, tag, value);
-    
-    return FALSE;
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+
+    success = gexiv2_metadata_try_set_tag_string(self, tag, value, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return success;
 }
 
-gchar* gexiv2_metadata_get_tag_interpreted_string (GExiv2Metadata *self, const gchar* tag) {
+gchar* gexiv2_metadata_try_get_tag_interpreted_string (GExiv2Metadata *self, const gchar* tag, GError **error) {
     g_return_val_if_fail(GEXIV2_IS_METADATA (self), NULL);
     g_return_val_if_fail(tag != NULL, NULL);
     g_return_val_if_fail(self->priv->image.get() != NULL, NULL);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_interpreted_string (self, tag);
+        return gexiv2_metadata_get_xmp_tag_interpreted_string (self, tag, error);
         
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_interpreted_string (self, tag);
+        return gexiv2_metadata_get_exif_tag_interpreted_string (self, tag, error);
         
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_interpreted_string (self, tag);
+        return gexiv2_metadata_get_iptc_tag_interpreted_string (self, tag, error);
     
     return NULL;
 }
 
-gchar** gexiv2_metadata_get_tag_multiple(GExiv2Metadata *self, const gchar* tag) {
-    g_return_val_if_fail(GEXIV2_IS_METADATA(self), NULL);
-    g_return_val_if_fail(tag != NULL, NULL);
-    g_return_val_if_fail(self->priv->image.get() != NULL, NULL);
-    
+gchar* gexiv2_metadata_get_tag_interpreted_string (GExiv2Metadata *self, const gchar* tag) {
+    gchar  *value;
+    GError *error = nullptr;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA (self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_interpreted_string(self, tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
+gchar** gexiv2_metadata_try_get_tag_multiple(GExiv2Metadata *self, const gchar* tag, GError **error) {
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
+
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_multiple(self, tag);
-    
+        return gexiv2_metadata_get_xmp_tag_multiple(self, tag, error);
+
+    if (gexiv2_metadata_is_exif_tag(tag))
+        return gexiv2_metadata_get_exif_tag_multiple(self, tag, error);
+
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_multiple(self, tag);
-    
+        return gexiv2_metadata_get_iptc_tag_multiple(self, tag, error);
+
     return NULL;
 }
 
-gboolean gexiv2_metadata_set_tag_multiple(GExiv2Metadata *self, const gchar* tag, const gchar** values) {
+gboolean gexiv2_metadata_try_set_tag_multiple(GExiv2Metadata *self, const gchar* tag, const gchar** values, GError **error) {
     g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
-    g_return_val_if_fail(tag != NULL, FALSE);
-    g_return_val_if_fail(values != NULL, FALSE);
-    g_return_val_if_fail(self->priv->image.get() != NULL, FALSE);
-    
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(values != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_set_xmp_tag_multiple(self, tag, values);
-    
+        return gexiv2_metadata_set_xmp_tag_multiple(self, tag, values, error);
+
+    if (gexiv2_metadata_is_exif_tag(tag))
+        return gexiv2_metadata_set_exif_tag_multiple(self, tag, values, error);
+
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_set_iptc_tag_multiple(self, tag, values);
-    
+        return gexiv2_metadata_set_iptc_tag_multiple(self, tag, values, error);
+
     return FALSE;
 }
 
-glong gexiv2_metadata_get_tag_long(GExiv2Metadata *self, const gchar* tag) {
+gchar** gexiv2_metadata_get_tag_multiple(GExiv2Metadata* self, const gchar* tag) {
+    gchar** tags = nullptr;
+    GError* error = nullptr;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
+
+    if (gexiv2_metadata_is_xmp_tag(tag))
+        tags = gexiv2_metadata_get_xmp_tag_multiple_deprecated(self, tag, &error);
+    else if (gexiv2_metadata_is_exif_tag(tag))
+        tags = gexiv2_metadata_get_exif_tag_multiple(self, tag, &error);
+    else if (gexiv2_metadata_is_iptc_tag(tag))
+        tags = gexiv2_metadata_get_iptc_tag_multiple(self, tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return tags;
+}
+
+gboolean gexiv2_metadata_set_tag_multiple(GExiv2Metadata *self, const gchar* tag, const gchar** values) {
+    GError   *error   = nullptr;
+    gboolean  success = FALSE;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(values != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+
+    success = gexiv2_metadata_try_set_tag_multiple(self, tag, values, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return success;
+}
+
+glong gexiv2_metadata_try_get_tag_long(GExiv2Metadata *self, const gchar* tag, GError **error) {
     g_return_val_if_fail(GEXIV2_IS_METADATA(self), 0);
     g_return_val_if_fail(tag != NULL, 0);
     g_return_val_if_fail(self->priv->image.get() != NULL, 0);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_long(self, tag);
+        return gexiv2_metadata_get_xmp_tag_long(self, tag, error);
     
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_long(self, tag);
+        return gexiv2_metadata_get_exif_tag_long(self, tag, error);
     
     return 0;
 }
 
-gboolean gexiv2_metadata_set_tag_long(GExiv2Metadata *self, const gchar* tag, glong value) {
+glong gexiv2_metadata_get_tag_long(GExiv2Metadata *self, const gchar* tag) {
+    GError *error   = nullptr;
+    glong   value;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+
+    value = gexiv2_metadata_try_get_tag_long(self, tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
+gboolean gexiv2_metadata_try_set_tag_long(GExiv2Metadata *self, const gchar* tag, glong value, GError **error) {
     g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
     g_return_val_if_fail(tag != NULL, FALSE);
     g_return_val_if_fail(self->priv->image.get() != NULL, 0);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_set_xmp_tag_long(self, tag, value);
+        return gexiv2_metadata_set_xmp_tag_long(self, tag, value, error);
     
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_set_exif_tag_long(self, tag, value);
+        return gexiv2_metadata_set_exif_tag_long(self, tag, value, error);
     
     return FALSE;
 }
 
+gboolean gexiv2_metadata_set_tag_long(GExiv2Metadata *self, const gchar* tag, glong value) {
+    GError   *error   = nullptr;
+    gboolean  success = FALSE;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA(self), FALSE);
+    g_return_val_if_fail(tag != nullptr, FALSE);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, FALSE);
+
+    success = gexiv2_metadata_try_set_tag_long(self, tag, value, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return success;
+}
+
 gboolean gexiv2_metadata_get_exposure_time (GExiv2Metadata *self, gint *nom, gint *den) {
-    return gexiv2_metadata_get_exif_tag_rational (self, "Exif.Photo.ExposureTime", nom, den);
+    return gexiv2_metadata_try_get_exif_tag_rational (self, "Exif.Photo.ExposureTime", nom, den, nullptr);
 }
 
 gdouble gexiv2_metadata_get_fnumber (GExiv2Metadata *self) {
@@ -1043,7 +1275,7 @@ gdouble gexiv2_metadata_get_focal_length (GExiv2Metadata *self) {
 }
 
 gint gexiv2_metadata_get_iso_speed (GExiv2Metadata *self) {
-    return (gint) gexiv2_metadata_get_exif_tag_long (self, "Exif.Photo.ISOSpeedRatings");
+    return (gint) gexiv2_metadata_get_exif_tag_long (self, "Exif.Photo.ISOSpeedRatings", NULL);
 }
 
 GExiv2PreviewProperties** gexiv2_metadata_get_preview_properties (GExiv2Metadata *self) {
@@ -1117,66 +1349,132 @@ void gexiv2_metadata_erase_exif_thumbnail (GExiv2Metadata *self) {
     thumb.erase();
 }
 
-const gchar* gexiv2_metadata_get_tag_label (const gchar *tag) {
-    g_return_val_if_fail(tag != NULL, NULL);
+const gchar* gexiv2_metadata_try_get_tag_label (const gchar *tag, GError **error) {
+    g_return_val_if_fail(tag != nullptr, nullptr);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_label(tag);
+        return gexiv2_metadata_get_xmp_tag_label(tag, error);
     
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_label(tag);
+        return gexiv2_metadata_get_exif_tag_label(tag, error);
     
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_label(tag);
+        return gexiv2_metadata_get_iptc_tag_label(tag, error);
     
-    return NULL;
+    return nullptr;
+}
+
+const gchar* gexiv2_metadata_get_tag_label (const gchar *tag) {
+    const gchar *value;
+    GError      *error = nullptr;
+
+    g_return_val_if_fail(tag != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_label(tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
+const gchar* gexiv2_metadata_try_get_tag_description (const gchar *tag, GError **error) {
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    
+    if (gexiv2_metadata_is_xmp_tag(tag))
+        return gexiv2_metadata_get_xmp_tag_description(tag, error);
+    
+    if (gexiv2_metadata_is_exif_tag(tag))
+        return gexiv2_metadata_get_exif_tag_description(tag, error);
+    
+    if (gexiv2_metadata_is_iptc_tag(tag))
+        return gexiv2_metadata_get_iptc_tag_description(tag, error);
+    
+    return nullptr;
 }
 
 const gchar* gexiv2_metadata_get_tag_description (const gchar *tag) {
+    const gchar *value;
+    GError      *error = nullptr;
+
+    g_return_val_if_fail(tag != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_description(tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
+const gchar* gexiv2_metadata_try_get_tag_type (const gchar *tag, GError **error) {
     g_return_val_if_fail(tag != NULL, NULL);
     
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_description(tag);
+        return gexiv2_metadata_get_xmp_tag_type(tag, error);
     
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_description(tag);
+        return gexiv2_metadata_get_exif_tag_type(tag, error);
     
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_description(tag);
+        return gexiv2_metadata_get_iptc_tag_type(tag, error);
     
     return NULL;
 }
 
 const gchar* gexiv2_metadata_get_tag_type (const gchar *tag) {
-    g_return_val_if_fail(tag != NULL, NULL);
-    
-    if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_type(tag);
-    
-    if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_type(tag);
-    
-    if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_type(tag);
-    
-    return NULL;
+    const gchar *value;
+    GError      *error = nullptr;
+
+    g_return_val_if_fail(tag != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_type(tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
 }
 
-GBytes* gexiv2_metadata_get_tag_raw(GExiv2Metadata *self, const gchar* tag) {
+GBytes* gexiv2_metadata_try_get_tag_raw(GExiv2Metadata *self, const gchar* tag, GError **error) {
     g_return_val_if_fail(GEXIV2_IS_METADATA (self), NULL);
     g_return_val_if_fail(tag != NULL, NULL);
     g_return_val_if_fail(self->priv->image.get() != NULL, NULL);
 
     if (gexiv2_metadata_is_xmp_tag(tag))
-        return gexiv2_metadata_get_xmp_tag_raw(self, tag);
+        return gexiv2_metadata_get_xmp_tag_raw(self, tag, error);
 
     if (gexiv2_metadata_is_exif_tag(tag))
-        return gexiv2_metadata_get_exif_tag_raw(self, tag);
+        return gexiv2_metadata_get_exif_tag_raw(self, tag, error);
 
     if (gexiv2_metadata_is_iptc_tag(tag))
-        return gexiv2_metadata_get_iptc_tag_raw(self, tag);
+        return gexiv2_metadata_get_iptc_tag_raw(self, tag, error);
 
     return NULL;
 }
 
+GBytes* gexiv2_metadata_get_tag_raw(GExiv2Metadata *self, const gchar* tag) {
+    GBytes *value;
+    GError *error = nullptr;
+
+    g_return_val_if_fail(GEXIV2_IS_METADATA (self), nullptr);
+    g_return_val_if_fail(tag != nullptr, nullptr);
+    g_return_val_if_fail(self->priv->image.get() != nullptr, nullptr);
+
+    value = gexiv2_metadata_try_get_tag_raw(self, tag, &error);
+
+    if (error) {
+        g_warning("%s", error->message);
+        g_clear_error(&error);
+    }
+
+    return value;
+}
+
 G_END_DECLS
